var statChart = {};
var baseURL = window.location.protocol + "//" + window.location.host + "/eccs";

$(document).ready(function() {
	$(".close").on("click", function() {
		$("#statisticsModal").hide()
	});
	
	$(document).on("click", "#statisticsButton", function() {
		$("#statisticsModal").show()
		getStatistics();
	})
});

function getStatistics() {
	showLoading();
	let reqData = {}
	reqData['dateFrom'] =  $.datepicker.formatDate("yy-mm-dd", $('#datepicker').datepicker().datepicker('getDate'));
	if ($("#eccstable thead th:last-child input").val() != '') {
		reqData['regAuth'] = $("#eccstable thead th:last-child input").val();
	}
	if ($("#eccstable thead th:nth-last-child(2) input").val() != '') {
		reqData['idp'] = $("#eccstable thead th:nth-last-child(2) input").val();
	}
	$.ajax({
		url: baseURL + "/api/webdata",
		type : 'GET',
		contentType: "application/json",
		data : reqData,
		success: function(result) {
			try {
				parsedData = $.parseJSON(result)
			} catch (err) {
				showMessage("Invalid data format received. Can't use to draw chart");
			}
		
			if ($.isEmptyObject(parsedData)) {
				showMessage("No data for parameters specified. Please refine query string.");
				return false;
			}
			fillInfo(parsedData);
		}, 
		error: function () {
			showMessage("Something happened. Can't retrieve data");
		}
	});
}
function showMessage(text) {
	hideModalContent();
	hideLoading();
	$("#statisticsModal .chart-error span").text(text)
	$("#statisticsModal .chart-error").css('display', 'block')
}
function hideMessage() {
	$("#statisticsModal .chart-error").css('display', 'none')
}
function showLoading() {
	hideModalContent();
	hideMessage();
	$("#statisticsModal .chart-loading").css('display', 'block')
}
function hideLoading() {
	$("#statisticsModal .chart-loading").css('display', 'none')
}
function fillInfo(parsedData) {
	let chartConfig = {};
	let tabledata = {};
	chartConfig["key"] = [];
	chartConfig["data1"] = [];
	chartConfig["data2"] = [];
	chartConfig["data3"] = [];
	$.each( parsedData, function( index, value ) {
		if (value.request_count === undefined || value.request_uniq === undefined
			|| value.request_count.api === undefined || value.request_count.web === undefined
			|| value.request_uniq.idp === undefined) {
			return true;
		}
		chartConfig["key"].push(index);
		chartConfig["data1"].push(value.request_count.api);
		chartConfig["data2"].push(value.request_count.web);
		chartConfig["data3"].push(value.request_uniq.idp);
		tabledata[index] = value.idp;
	});
	if (!chartConfig["key"].length) {
		showMessage("No valid data for parameters specified. Please refine query string.");
		return false;
	}
	drawChart(chartConfig);
	fillTable(tabledata);
	hideMessage();
	hideLoading();
	showModalContent();
}
function drawChart(chartData) {
	let chartType = 'line';
	let dataSet = [];
	let key = [];
	if (chartData["key"].length == 1) {
		chartType = 'doughnut';
		dataSet = [
			{ label: chartData["key"][0], data: [chartData["data1"][0], chartData["data2"][0], chartData["data3"][0]] },
		];
		key = ['API', 'WEB', 'Unique IdP'];
	} else {
		dataSet = [
			{ label: 'API', data: chartData["data1"] },
			{ label: 'WEB', data: chartData["data2"]	},
			{ label: 'Unique IdP', data: chartData["data3"] }
		];
		key = chartData["key"];
	}
	let options = {
		type: chartType,
		data: {
			labels: key,
			datasets: dataSet
		},
		options: {
			responsive: true,
			maintainAspectRatio: false,
			interaction: {
				mode: 'index',
				intersect: false,
			}
		}
	}
	var ctx = document.getElementById('statistics').getContext('2d');
	if (!$.isEmptyObject(statChart) && statChart.config.type != chartType) {
		statChart.destroy();
		statChart = {};
	}
	if ($.isEmptyObject(statChart)) {
		statChart = new Chart(ctx, options);
	} else {
		statChart.data.datasets = dataSet;
		statChart.data.labels = key;
		statChart.update();
	}
}
function fillTable(tabledata) {
	let container = $(".data-container")
	container.html("");
	$.each(tabledata, function (index, value) {
		let list = '<ul style="list-style-type: decimal;">';
		if (Object.keys(value).length > 0) {
			$.each(value, function(idp, count) {
				list += "<li>"+idp+":<strong>"+count+"</strong></li>"
			})
		} else {
			list += 'No data';
		}
		list += "</ul>"
		let button  = document.createElement("button")
		button.classList.add("collapsible");
		button.textContent = index;
		button.addEventListener("click", function() {
			this.classList.toggle("active");
			var content = this.nextElementSibling;
			
			if (content.style.maxHeight){
				content.style.maxHeight = null;
			} else {
				content.style.maxHeight = content.scrollHeight + "px";
			} 
		});
		let div  = document.createElement("div")
		div.classList.add("content");
		div.innerHTML=list; 
		container.append(button)
		container.append(div)
	})
}
function hideModalContent() {
	$("#statisticsModal .chart-container").css('display', 'none')
	$("#statisticsModal .data-container").css('display', 'none')
}
function showModalContent() {
	$("#statisticsModal .chart-container").css('display', 'block')
	$("#statisticsModal .data-container").css('display', 'block')
}
